/** @file
 * @brief Config class implementation. This class represents the actual configuration.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/


#include "config.h"


/**
 * Default Constructor. Available command line and config file options with their default values are defined here.
 */
Config::Config(LoggerPtr _log)
    : Daemon_mode(false)
    , Loglevel(0)
    , Syslog(false)
{
    // initialize Logger
    Log = _log;

    // Available service description config options
    po::options_description opt_desc_service("Service description options");
    opt_desc_service.add_options()
        ("protocol",po::value<string>(),"The service protocol.")
        ("host",po::value<string>(),"The hostname to update.")
        ("login",po::value<string>(),"Login name.")
        ("password",po::value<string>(),"Corresponding password.")
    ;

    // Available command line only options
    po::options_description opt_desc_cmd_only("Command line only options");
    opt_desc_cmd_only.add_options()
        ("help,?","Show help.")
        ("version,v","Show version.")
        ("config,c",po::value<string>()->default_value("/etc/bpdyndnsd"),"Set the config path.")
    ;

     // Available generic options. Valid on cmd or in config file.
    po::options_description opt_desc_generic("Generic config options");
    opt_desc_generic.add_options()
        ("daemon_mode",po::value<bool>()->default_value(false),"Run as system daemon.")
        ("logfile",po::value<string>()->default_value("/var/log/bpdyndns.log"),"Where to log.")
        ("loglevel",po::value<int>()->default_value(0),"Loglevel.")
        ("syslog",po::value<bool>()->default_value(false),"Use syslog facility.")
    ;

    // Define valid command line parameters
    Opt_desc_cmd = new po::options_description("Command line options");
    Opt_desc_cmd->add(opt_desc_cmd_only);
    Opt_desc_cmd->add(opt_desc_generic);
    Opt_desc_cmd->add(opt_desc_service);

    // Define valid config file options
    Opt_desc_conf_main = new po::options_description("Config file options");
    Opt_desc_conf_main->add(opt_desc_generic);

    // Define valid service file options
    Opt_desc_conf_service = new po::options_description("Service file options");
    Opt_desc_conf_service->add(opt_desc_service);

    Log->print_constructor_call("Config");
}


/**
 * Default Destructor
 */
Config::~Config()
{
    delete Opt_desc_cmd;
    delete Opt_desc_conf_main;
    delete Opt_desc_conf_service;

    Log->print_destructor_call("Config");
}


/**
 * Parses the command line arguments and does the needed actions.
 * @param argc Command line argument number given to main.
 * @param argv[] Pointer to command line argument array given to main.
 * @return 0 if all is fine, 1 if not.
 */
int Config::parse_cmd_line(int argc, char *argv[])
{
    try
    {
        po::store(po::parse_command_line(argc, argv, *this->Opt_desc_cmd), Variables_map);
        po::notify(Variables_map);

        if ( Variables_map.count("help") )
        {
            Log->print_usage(Opt_desc_cmd);
            return 1;
        }
        else if ( Variables_map.count("version") )
        {
            Log->print_version();
            return 1;
        }

        // Create a service object if all needed options are set on the command line
        if ( Variables_map.count("protocol") && Variables_map.count("host") && Variables_map.count("login") && Variables_map.count("password") )
        {
            // Get the cmd parameter values for protocol host login and password
            string protocol = Variables_map["protocol"].as<string>();
            string host = Variables_map["host"].as<string>();
            string login = Variables_map["login"].as<string>();
            string password = Variables_map["password"].as<string>();

            //TODO: convert protocol option to lowercase

            ServicePtr service = create_service(protocol,host,login,password);
            if ( service != NULL )
                Services.push_back(service);
            else
                return 1;
        }
        else if ( Variables_map.count("protocol") || Variables_map.count("host") || Variables_map.count("login") || Variables_map.count("password") )
        {
            Log->print_missing_cmd_service_option();
            Log->print_usage(Opt_desc_cmd);
            return 1;
        }

        if ( Variables_map.count("config") )
        {
            fs::path full_config_path = fs::system_complete(fs::path(Variables_map["config"].as<string>()));
            Config_path = full_config_path.string();
            if ( !fs::exists(full_config_path) ||  !fs::is_directory(full_config_path) )
            {
                // Config path doesn't exist or is not a directory
                Log->print_error_config_path(Config_path);
                return 1;
            }
        }
    }
    catch(po::unknown_option e)
    {
        Log->print_unknown_cmd_option(e.what());
        Log->print_usage(Opt_desc_cmd);
        return 1;
    }
    return 0;
}


/**
 * 
 * @param protocol 
 * @param host 
 * @param login 
 * @param password 
 * @return A pointer to the created Service object.
 */
ServicePtr Config::create_service(const string &protocol,const string &host, const string &login, const string &password)
{
    if(protocol == "dhs")
    {
        ServicePtr service_dhs(new DHS(Log,host,login,password));
        return service_dhs;
    }
    else if(protocol == "ods")
    {
        ServicePtr service_ods(new ODS(Log,host,login,password));
        return service_ods;
    }
    else
    {
        Log->print_unknown_protocol(protocol);
        ServicePtr service;
        return service;
    }
}


/**
 * Loads a service config file, invoked by load_config_from_files.
 * @param full_filename Filename of the service config file to load.
 * @return 0 if all is fine, 3 if an unknown option was detected, 4 if the service file could not be opened for reading.
 */
int Config::load_service_config_file(const string& full_filename)
{
    Log->print_load_service_conf(full_filename);

    ifstream service_config_file(full_filename.c_str(),ifstream::in);
    if(service_config_file.is_open())
    {
        try
        {
            po::variables_map vm;
            po::parsed_options parsed_service_options = po::parse_config_file(service_config_file,*this->Opt_desc_conf_service,true);
            po::store(parsed_service_options,vm);
            po::notify(vm);

            if(vm.count("protocol") && vm.count("host") && vm.count("login") && vm.count("password"))
            {
                // create the corresponding service
                string protocol = vm["protocol"].as<string>();
                string host = vm["host"].as<string>();
                string login = vm["login"].as<string>();
                string password = vm["password"].as<string>();

                // TODO: convert protocol to lowercase
                //protocol = tolower(protocol.c_str());

                ServicePtr service = create_service(protocol,host,login,password);
                if ( service != NULL )
                {
                    Services.push_back(service);
                }
            }
        }
        catch ( po::unknown_option e )
        {
            // unknown option in config file detected
            service_config_file.close();
            Log->print_unknown_service_conf_option(e.what());
            return 1;
        }
        service_config_file.close();
    }
    else
    {
        // error opening service config file for reading
        Log->print_error_opening(full_filename);
        return 1;
    }
    return 0;
}


/**
 * Loads the main config file, invoked by load_config_from_files
 * @param full_filename The full filename of the main config file to load
 * @return 0 if all is fine. 3 if unknown option was detected, 4 if main config file could not be opened for reading
 */
int Config::load_main_config_file(const string& full_filename)
{
    Log->print_load_main_conf(full_filename);

    ifstream main_config_file(full_filename.c_str(),ifstream::in);
    if(main_config_file.is_open())
    {
        try
        {
            po::parsed_options parsed_main_options = po::parse_config_file(main_config_file,*this->Opt_desc_conf_main,true);
            po::store(parsed_main_options,Variables_map);
            po::notify(Variables_map);

            if(Variables_map.count("daemon_mode") && Variables_map.count("logfile") && Variables_map.count("loglevel") && Variables_map.count("syslog"))
            {
                Daemon_mode = Variables_map["daemon_mode"].as<bool>();
                Logfile = Variables_map["logfile"].as<string>();
                Loglevel = Variables_map["loglevel"].as<int>();
                Syslog = Variables_map["syslog"].as<bool>();
            }
        }
        catch ( po::unknown_option e )
        {
            // unknown option in main config file detected
            main_config_file.close();
            Log->print_unknown_main_conf_option(e.what());
            return 1;
        }
        main_config_file.close();
    }
    else
    {
        // error opening main config file for reading
        Log->print_error_opening(full_filename);
        return 1;
    }
    return 0;
}


/**
 * Loads the main and the service config file and does the needed action.
 * @param config_path The path to the config directory.
 * @return 0 if all is fine.
 */
int Config::load_config_from_files()
{
    fs::path full_config_path = fs::path(Config_path);

    fs::directory_iterator end_iter;
    for ( fs::directory_iterator dir_itr(full_config_path) ; dir_itr != end_iter ; ++dir_itr )
    {
        if( fs::is_regular_file( dir_itr->status() ) )
        {
            string actual_file = dir_itr->path().filename();
            boost::regex expr(".*\.conf?");
             // If it is the main config file do the following
            if ( actual_file == "bpdyndnsd.conf" )
            {
                // Load the main config file
                string full_filename = dir_itr->path().string();
                if ( load_main_config_file(full_filename) != 0 )
                    return 1;
            }
            // If it is a service definition file *.conf, parse it and generate the corresponding service
            else if ( boost::regex_search( actual_file,expr ) )
            {
                string full_filename = dir_itr->path().string();
                if ( load_service_config_file(full_filename) != 0 )
                    return 1;
            }
        }
    }
    // Config file successfully loaded
    Log->print_conf_loaded(Config_path);
    return 0;
}


/**
 * Getter method for Service list member.
 * @return Pointer to a list of Service's.
 */
list<ServicePtr> Config::get_services()
{
    return this->Services;
}


/**
 * Getter method for member Opt_desc_cmd.
 * @return options_description*.
 */
po::options_description* Config::get_opt_desc_cmd()
{
    return Opt_desc_cmd;
}


/**
 * Getter method for member Opt_desc_conf_main.
 * @return options_description*.
 */
po::options_description* Config::get_opt_desc_conf_main()
{
    return Opt_desc_conf_main;
}


/**
 * Getter method for member Opt_desc_conf_service.
 * @return options_description*.
 */
po::options_description* Config::get_opt_desc_conf_service()
{
    return Opt_desc_conf_service;
}


/**
 * Getter for member Loglevel.
 * @return Member Loglevel.
 */
int Config::get_loglevel()
{
    return Loglevel;
}


/**
 * Getter for member Daemon_mode.
 * @return TRUE if enabled, FALSE if disabled.
 */
bool Config::get_daemon_mode()
{
    return Daemon_mode;
}


/**
 * Resets all shared Service pointers and clears the Services list.
 */
void Config::delete_services()
{
    BOOST_FOREACH( ServicePtr service, Services )
    {
        service.reset();
    }
    Services.clear();
}
