/** @file
 * @brief Logger class implementation. This class implements the Logging facility.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/


#include "logger.h"


/**
 * Default Constructor
 */
Logger::Logger()
{
    print_constructor_call("Logger");
}


/**
 * Default Destructor
 */
Logger::~Logger()
{
    print_destructor_call("Logger");
}


/**
 * Prints out the usage to the command line.
 */
void Logger::print_usage(const po::options_description* opt_desc)
{
    cout << "Usage: bpdyndnsd [Command line options]" << "\n" << endl;
    cout << *opt_desc << endl;
}


/**
 * Prints out the programm name and the given version string on stdout.
 * @param version Version string.
 */
void Logger::print_version()
{
    ostringstream version_string;
    version_string << VERSION << "." << REVISION << "." << RELEASE;
    cout << "Bullet proof dynamic dns daemon.\nbpdyndnsd " << version_string.str() << endl;
}


/**
 * Prints out the successful parsing of the command line options.
 */
void Logger::print_cmd_parsed()
{
    cout << "Command line options successfully parsed." << endl;
}


/**
 * Prints out the successful parsing of the config files.
 * @param config_path The specified config path.
 */
void Logger::print_conf_loaded(const string& config_path)
{
    cout << "Config files successfully loaded in: " << config_path << endl;
}


/**
 * Prints out the unsuccessful parsing of the config files.
 * @param config_path The specified config path.
 */
void Logger::print_conf_not_loaded(const string& config_path)
{
    cerr << "Config files couldn't be loaded in: " << config_path << endl;
}


/**
 * A file could not be opened for reading.
 * @param filename The filename.
 */
void Logger::print_error_opening(const string& filename)
{
    cerr << "Error opening file for reading: " << filename << endl;
}


/**
 * Desctructor of specified class was called.
 * @param _class Name of the class.
 */
void Logger::print_destructor_call(const string& _class)
{
    cout << "Destructor call: " << _class << endl;
}


/**
 * Constructor of specified class was called.
 * @param _class Name of the class.
 */
void Logger::print_constructor_call(const string& _class)
{
    cout << "Constructor call: " << _class << endl;
}


/**
 * Update method for specified service was called.
 * @param service The service for which is the update running.
 */
void Logger::print_update_service(const string& service)
{
    cout << "Running update for service: " << service << endl;
}


/**
 * An unknown option on the command line was detected.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_cmd_option(const string& unknown_option)
{
    cerr << "Unknown option on command line detected: " << unknown_option << endl;
}


/**
 * An unknown protocol was specified.
 * @param protocol The unknown protocol.
 */
void Logger::print_unknown_protocol(const string& protocol)
{
    cerr << "Unknown protocol defined: " << protocol << endl;
}


/**
 * Loading a service config file.
 * @param filename The service config file.
 */
void Logger::print_load_service_conf(const string& filename)
{
    cout << "Loading service config file: " << filename << endl;
}


/**
 * Loading the main config file.
 * @param filename The main config file.
 */
void Logger::print_load_main_conf(const string& filename)
{
    cout << "Loading main config file: " << filename << endl;
}


/**
 * There is an unknown option in a service config file.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_service_conf_option(const string& unknown_option)
{
    cerr << "Unknown option in service config file detected: " << unknown_option << endl;
}


/**
 * There is an unknown option in the main config file.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_main_conf_option(const string& unknown_option)
{
    cerr << "Unknown option in main config file detected: " << unknown_option << endl;
}


/**
 * The defined config path doesn't exist or is not a directory.
 * @param config_path The defined config path.
 */
void Logger::print_error_config_path(const string& config_path)
{
    cerr << "Config path doesn't exists or is not a diretory: " << config_path << endl;
}


/**
 * There is a missing command line option to initialize a service object.
 */
void Logger::print_missing_cmd_service_option()
{
    cerr << "Missing option to initialize service. Protocol, host, login and password must be specified." << endl;
}


/**
 * Process running as daemon.
 * @param pid The pid of the daemon.
 */
void Logger::print_runnig_as_daemon(const int pid)
{
    cout << "Runnig as daemon: " << pid << endl;
}


/**
 * Prints out the daemon mode.
 * @param daemon_mode The daemon mode.
 */
void Logger::print_daemon_mode(const bool daemon_mode)
{
    string mode = "disabled";
    if (daemon_mode == true)
        mode = "enabled";
    cout << "Daemon mode is " << mode << "." << endl;
}


/**
 * There was an error while trying to fork.
 */
void Logger::print_error_fork()
{
    cerr << "Error while trying to fork." << endl;
}


/**
 * A pid in the pidfile was found.
 * @param pid The pid found in the pidfile.
 */
void Logger::print_pid_found(const int pid)
{
    cout << "Pidfile found: " << pid << ". Checking if process is still runnig..." << endl;
}


/**
 * Another process is already running.
 * @param pid The pid of the other process.
 */
void Logger::print_process_already_running(const int pid)
{
    cerr << "Another bpdyndnsd process with PID " << pid << " is already running!" << endl;
}


/**
 * SIGTERM caught.
 */
void Logger::print_caught_sigterm()
{
    cout << "Caught SIGTERM. Exiting..." << endl;
}


/**
 * SIGUSR1 caught.
 */
void Logger::print_caught_siguser1()
{
    cout << "Caught SIGUSR1. Switching to offline mode..." << endl;
}


/**
 * SIGHUP caught.
 */
void Logger::print_caught_sighup()
{
    cout << "Caught SIGHUP. Reloading config and switching to online mode..." << endl;
}
