/** @file
 * @brief The main function.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define VERSION     0
#define REVISION    1
#define RELEASE     0

#define PIDFILE "/var/run/bpdyndnsd.pid"

#include <iostream>
#include <list>
#include <string>

#include <boost/foreach.hpp>

#include "updater.cpp"
#include "config.cpp"
#include "logger.cpp"

#include "service.cpp"

#include "dhs.cpp"
#include "ods.cpp"

#include <sys/types.h>
#include <signal.h>


using namespace std;

typedef boost::shared_ptr<Updater> UpdaterPtr;

UpdaterPtr updater;
bool online_mode = 1;

/**
 * Checks if a bpdyndnsd process is already running.
 * @param updater Shared Pointer to updater, needed for logging.
 * @return 0 if process not running already or pid of already running process.
 */
int check_for_running_process()
{
    ifstream pidfile(PIDFILE);
    if ( pidfile.is_open() )
    {
        int pid;
        pidfile >> pid;
        if ( pid > 0 )
            updater->get_logger()->print_pid_found(pid);
        else
            return 0;

        // check if process still running ret_val==-1 -> not runnig, ret_val==0 -> running
        if ( kill(pid,0) == 0)
        {
            updater->get_logger()->print_process_already_running(pid);
            pidfile.close();
            return pid;
        }
    }
    pidfile.close();
    return 0;
}


/**
 * Writes the pid into the pidfile.
 * @param pid The process's pid.
 */
void write_pidfile(int pid)
{
    ofstream pidfile(PIDFILE);
    if ( pidfile.is_open() )
    {
        pidfile << pid << endl;
    }
    pidfile.close();
}


/**
 * Signal SIGTERM caught, releasing resources and exit.
 * @param param Parameter from the signal interface.
 */
void terminate(int param)
{
    updater->get_logger()->print_caught_sigterm();
    updater.reset();
    exit(1);
}


/**
 * Signal SIGUSR1 caught, switching to offline mode.
 * @param param Parameter from the signal interface.
 */
void switch_to_offline(int param)
{
    online_mode = 0;
}


/**
 * Signal SIGHUP caught, reloading config and switching to online mode.
 * @param param 
 */
void reload_config(int param)
{
    updater->reload_config();
    online_mode = 1;
}


/**
 * Initialize the signals we handle.
 */
void init_signals()
{
    signal(SIGTERM,terminate);
    signal(SIGUSR1,switch_to_offline);
    signal(SIGHUP,reload_config);
}


/**
 * @brief The main part.
 * @param argc Number of arguments
 * @param argv Command line arguments
 * @return 0 if all is fine.
 */
int main(int argc, char *argv[])
{
    // initialize Updater
    UpdaterPtr _updater(new Updater);
    updater = _updater;
    _updater.reset();

    // load the cmd options
    if ( updater->init_config_from_cmd(argc,argv) != 0 )
        return 0;

    // load the config and service files
    if ( updater->init_config_from_files() != 0 )
        return 0;

    // open pidfile and check for running process
    if ( check_for_running_process() != 0)
        return 0;

    // init signal handling
    init_signals();

    // set the configured loggin facility, default stdout

    // initialize daemon mode if configured
    bool daemon_mode = updater->get_config()->get_daemon_mode();
    updater->get_logger()->print_daemon_mode(daemon_mode);

    // fork if daemon mode is enabled
    if ( daemon_mode == 1 )
    {
        int pid = fork();
        if ( pid < 0 )
        {
            // error fork
            updater->get_logger()->print_error_fork();
            return 0;
        }
        else if ( pid > 0 )
        {
            // parent
            write_pidfile(pid);
            updater->get_logger()->print_runnig_as_daemon(pid);
            return 0;
        }
        // child starts here
    }

    // service processing starts here
    do
    {
        if ( online_mode == 1 )
        {
            // update all configured services
            updater->update_services();
        }
        sleep(2);

    }while ( daemon_mode == 1 );

    return 0;
}
