/** @file
 * @brief The updater class implementation. This class implements the updater logic.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#include "updater.h"

#include <boost/foreach.hpp>


/**
 * Default constructor which initializes the member Conf.
 */
Updater::Updater()
{
    // initialize Logger
    LoggerPtr _log(new Logger);
    Log = _log;

    // initialize Config
    ConfigPtr _config(new Config(Log));
    Conf = _config;

    Log->print_constructor_call("Updater");
}


/**
 * Default destructor.
 */
Updater::~Updater()
{
    Log->print_destructor_call("Updater");
}


/**
 * Parse the command line arguments and initialize corresponding options.
 * @param argc Number command line arguments.
 * @param argv[] Array with arguments.
 * @return 0 if cmd options successfully parsed, 1 if usage or version.
 */
int Updater::init_config_from_cmd(int argc, char *argv[])
{
    // Load the command line parameters
    if( Conf->parse_cmd_line( argc, argv ) != 0)
        return 1;

    // successful parsed
    Log->print_cmd_parsed();
    return 0;
}


/**
 * Load the main config and the service definition files in config path.
 * @return 0 if all is fine, 
 */
int Updater::init_config_from_files()
{
    // Load the main and service config files in config path
    if( Conf->load_config_from_files() != 0)
        return 1;

    // successful loaded
    return 0;
}


/**
 * Getter for member Config.
 * @return Member Config.
 */
ConfigPtr Updater::get_config()
{
    return Conf;
}


/**
 * Getter for member Logger.
 * @return Member Logger.
 */
LoggerPtr Updater::get_logger()
{
    return Log;
}


/**
 * Reloading the config. Delete all Service objects and then init new Service objects from config files.
 */
void Updater::reload_config()
{
    Conf->delete_services();
    init_config_from_files();
}


/**
 * Update all configured services.
 */
void Updater::update_services()
{
    list<ServicePtr> services = this->Conf->get_services();

    string ip = "192.168.1.1";

    BOOST_FOREACH( ServicePtr service, services )
    {
        service->update(ip);
    }
}
